package istia.st.springmvc.personnes.tests;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Iterator;
import org.springframework.beans.factory.xml.XmlBeanFactory;
import org.springframework.core.io.ClassPathResource;

import istia.st.springmvc.personnes.dao.DaoException;
import istia.st.springmvc.personnes.dao.IDao;
import istia.st.springmvc.personnes.entites.Personne;
import junit.framework.TestCase;

public class TestDaoSqlExpress extends TestCase {

	// couche [dao]
	private IDao dao;

	public IDao getDao() {
		return dao;
	}

	public void setDao(IDao dao) {
		this.dao = dao;
	}

	// constructeur
	public void setUp() {
		dao = (IDao) (new XmlBeanFactory(new ClassPathResource(
				"spring-config-test-dao-sqlexpress.xml"))).getBean("dao");
	}

	// liste des personnes
	private void doListe(Collection personnes) {
		Iterator iter = personnes.iterator();
		while (iter.hasNext()) {
			System.out.println(iter.next());
		}
	}

	// test1
	public void test1() throws ParseException {
		// liste actuelle
		Collection personnes = dao.getAll();
		int nbPersonnes = personnes.size();
		// affichage
		doListe(personnes);
		// ajout d'une personne
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 1);
		dao.saveOne(p1);
		int id1 = p1.getId();
		// vrification - on aura un plantage si la personne n'est pas trouve
		p1 = dao.getOne(id1);
		assertEquals("X", p1.getNom());
		// modification
		p1.setNom("Y");
		dao.saveOne(p1);
		// vrification - on aura un plantage si la personne n'est pas trouve
		p1 = dao.getOne(id1);
		assertEquals("Y", p1.getNom());
		// suppression
		dao.deleteOne(id1);
		// vrification
		int codeErreur = 0;
		boolean erreur = false;
		try {
			p1 = dao.getOne(id1);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// liste des personnes
		personnes = dao.getAll();
		assertEquals(nbPersonnes, personnes.size());
	}

	// modification-suppression d'un lment inexistant
	public void test2() throws ParseException {
		// d'abord ajout
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 1);
		dao.saveOne(p1);
		// rcupration copie de la personne
		Personne p2 = dao.getOne(p1.getId());
		// modification copie avec un id inexistant
		p2.setId(p1.getId() + 1);
		// modification nom de la copie
		p2.setNom("Y");
		// sauvegarde copie - on doit avoir une DaoException de code 2
		boolean erreur = false;
		int codeErreur = 0;
		try {
			dao.saveOne(p2);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// vrification - on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// suppression lment inexistant - on doit avoir une DaoException de
		// code 2
		erreur = false;
		codeErreur = 0;
		try {
			dao.deleteOne(-2);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// vrification - on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
		// suppression personne p1
		dao.deleteOne(p1.getId());
		// vrification
		try {
			p1 = dao.getOne(p1.getId());
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
	}

	// gestion des versions de personne
	public void test3() throws ParseException, InterruptedException {
		// d'abord ajout
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 1);
		dao.saveOne(p1);
		// rcupration copie p2 de la personne p1
		Personne p2 = dao.getOne(p1.getId());
		// rcupration copie p3 de la personne p1
		Personne p3 = dao.getOne(p1.getId());
		// on vrifie qu'on a bien la mme version
		assertEquals(p2.getVersion(), p3.getVersion());
		// attente 10 ms
		Thread.sleep(10);
		// sauvegarde copie p2 - la version de p1 va changer
		dao.saveOne(p2);
		// sauvegarde copie p3 - on doit avoir une DaoException de code 2
		boolean erreur = false;
		int codeErreur = 0;
		try {
			dao.saveOne(p3);
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// vrification - on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(codeErreur, 2);
		// suppression personne p1
		dao.deleteOne(p1.getId());
		// vrification
		try {
			p1 = dao.getOne(p1.getId());
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
	}

	// optimistic locking - accs multi-threads
	public void test4() throws Exception {
		// ajout d'une personne
		Personne p1 = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 0);
		dao.saveOne(p1);
		int id1 = p1.getId();
		// cration de N threads de mise  jour du nombre d'enfants
		final int N = 100;
		Thread[] taches = new Thread[N];
		for (int i = 0; i < taches.length; i++) {
			taches[i] = new ThreadDaoMajEnfants("thread n " + i, dao, id1);
			taches[i].start();
		}
		// on attend la fin des threads
		for (int i = 0; i < taches.length; i++) {
			taches[i].join();
		}
		// on rcupre la personne
		p1 = dao.getOne(id1);
		// elle doit avoir N enfants
		assertEquals(N, p1.getNbEnfants());
		// suppression personne p1
		dao.deleteOne(p1.getId());
		// vrification
		boolean erreur = false;
		int codeErreur = 0;
		try {
			p1 = dao.getOne(p1.getId());
		} catch (DaoException ex) {
			erreur = true;
			codeErreur = ex.getCode();
		}
		// on doit avoir une erreur de code 2
		assertTrue(erreur);
		assertEquals(2, codeErreur);
	}

	// tests de validit de saveOne
	public void test5() throws ParseException {
		int codeErreur = 0;
		// personne null
		try {
			dao.saveOne(null);
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(10, codeErreur);
		// id erron
		try {
			dao.saveOne(new Personne(-2, "X", "X", new SimpleDateFormat(
					"dd/MM/yyyy").parse("01/02/2006"), true, 0));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(11, codeErreur);
		// date de naissance manquante
		try {
			dao.saveOne(new Personne(100, "X", "X", null, true, 0));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(12, codeErreur);
		// nombre d'enfants invalide
		try {
			dao.saveOne(new Personne(100, "X", "X", new SimpleDateFormat(
					"dd/MM/yyyy").parse("01/02/2006"), true, -2));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(13, codeErreur);
		// nom manquant
		try {
			dao.saveOne(new Personne(100, "X", "    ", new SimpleDateFormat(
					"dd/MM/yyyy").parse("01/02/2006"), true, 2));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(14, codeErreur);
		// prnom manquant
		try {
			dao.saveOne(new Personne(100, "  ", "X", new SimpleDateFormat(
					"dd/MM/yyyy").parse("01/02/2006"), true, 2));
		} catch (DaoException ex) {
			codeErreur = ex.getCode();
		}
		assertEquals(15, codeErreur);
	}

	// insertions multi-threads
	public void test6() throws ParseException, InterruptedException{
		// cration d'une personne
		Personne p = new Personne(-1, "X", "X", new SimpleDateFormat(
				"dd/MM/yyyy").parse("01/02/2006"), true, 0);
		// qu'on duplique N fois dans un tableau
		final int N = 10;
		Personne[] personnes=new Personne[N];
		for(int i=0;i<personnes.length;i++){
			personnes[i]=new Personne(p);
		}
		// cration de N threads d'insertion - chaque thread insre 1 personne
		Thread[] taches = new Thread[N];
		for (int i = 0; i < taches.length; i++) {
			taches[i] = new ThreadDaoInsertPersonne("thread n " + i, dao, personnes[i]);
			taches[i].start();
		}
		// on attend la fin des threads
		for (int i = 0; i < taches.length; i++) {
			// thread n i
			taches[i].join();
			// supression personne
			dao.deleteOne(personnes[i].getId());
		}
	}
}